/*
 *	Nano-X sample program
 *
 *	Autor : Mathias Langer
 *			taskit GmbH
 *			www.taskit.de
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <nano-X.h>
#include <nxcolors.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/input.h>

#define W1_WIDTH	196
#define W1_HEIGHT	182

/* if in indexed mode disable for speed */
#define __PAINTCOLORAREAS__

/*
	simple object system
*/

/*
	gObject - basic object structure
	callbacks : onPaint - to repaint the object
	            onUp    - called at released mouse button
	            onDown  - called at pressed mouse button
	            onMove  - called at mouse motion
	            onExit  - called when mouse leaves object
	Data - points to sub-object structure
*/
struct gObject
{
	int Type;
	int xPos;
	int yPos;
	int Width;
	int Height;
	void (*onPaint)(struct gObject *Object);
	void (*onUp)(GR_EVENT event, struct gObject *Object);
	void (*onDown)(GR_EVENT event, struct gObject *Object);
	void (*onMove)(GR_EVENT event, struct gObject *Object);
	void (*onExit)(GR_EVENT event, struct gObject *Object);
	void *Data;
};

/*
	list element containing an object
*/
struct lElement
{
	struct lElement *next;
	struct lElement *prev;
	struct gObject *Object;
};

/*
	deletes an object
*/
void delObject(struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data != NULL) free(Object->Data);
	free(Object);
	Object = NULL;
}

/*
	adds an element to object list
*/
void addElement(struct lElement *prev, struct lElement *Element)
{
	void *next;

	if((prev == NULL) || (Element == NULL)) return;
	next = prev->next;
	prev->next = Element;
	Element->next = next;
	Element->prev = prev;
}

/*
	removes an element from object list - without deleting it !!!
*/
void subElement(struct lElement *Position)
{
	if(Position == NULL) return;
	if(Position->prev != NULL) Position->prev->next = Position->next;
	if(Position->next != NULL) Position->next->prev = Position->prev;
	Position->prev = NULL;
	Position->next = NULL;
}

/*
	multiplicates r, g, b components of Color with faktor
	faktor is a fix-comma number (8.8)
	helper function for button borders
*/
int mulColor(int Color, unsigned short faktor)
{
	unsigned short tmp;
	int retval = 0;
	int i;

	for(i=0;i<3;i++)
	{
		tmp = (Color & 0xFF) << 8;
		tmp = (tmp * faktor) >> 8;
		retval = retval | ((tmp >> 8) << (i*8));
		Color = Color >> 8;
	}

	return(retval);
}

/*
	types of graphic objects
*/

#define TYPE_BUTTON		1
#define TYPE_SLIDER		2
#define TYPE_COLORBAR	3
#define TYPE_COLORAREA	4

/*
	sub object structure for simple buttons
	callbacks : Action - called by button press
*/
struct gButton
{
	char *Caption;
	int Length;
	int ForeGroundColor;
	int BackGroundColor;
	int down;
	void (*Action)(GR_EVENT event, struct gObject *Object);
};

/*
	sub object structure for sliders
*/
struct gSlider
{
	char *Caption;
	int ForeGroundColor;
	int BackGroundColor;
	int Min;
	int Max;
	int Value;
};

/*
	sub object structure for colored bars
*/
struct gColorBar
{
	int lColor;
	int rColor;
};

/*
	sub object structure for colored areas
*/
struct gColorArea
{
	int ulColor;
	int urColor;
	int llColor;
	int lrColor;
};

/*
	paint functions
*/

GR_WINDOW_ID w1;
GR_GC_ID gc1;
struct lElement root;
struct gObject *ColorBarR;
struct gObject *ColorBarG;
struct gObject *ColorBarB;
struct gObject *ColorArea1;
struct gObject *ColorArea2;
int exitProgram = 0;

/*
	slider paint function
*/
void paintSlider(struct gObject *Object)
{
	int Pos, Min, Max, Value;

	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	GrSetGCForeground(gc1, ((struct gSlider *)(Object->Data))->BackGroundColor);
	GrFillRect(w1, gc1, Object->xPos, Object->yPos, Object->Width, Object->Height);
	GrSetGCForeground(gc1, ((struct gSlider *)(Object->Data))->ForeGroundColor);
	GrRect(w1, gc1, Object->xPos, Object->yPos, Object->Width, Object->Height);
	Value = ((struct gSlider *)(Object->Data))->Value;
	Min = ((struct gSlider *)(Object->Data))->Min;
	Max = ((struct gSlider *)(Object->Data))->Max;
	Pos = ((Object->Width - 3) * (Value - Min)) / (Max - Min) + Object->xPos + 1;
	GrLine(w1, gc1, Pos - 1, Object->yPos, Pos - 1, Object->yPos + Object->Height - 1);
	GrLine(w1, gc1, Pos, Object->yPos, Pos, Object->yPos + Object->Height - 1);
	GrLine(w1, gc1, Pos + 1, Object->yPos, Pos + 1, Object->yPos + Object->Height - 1);
}

/*
	button paint function
*/
void paintButton(struct gObject *Object)
{
	int Border_dark;
	int Border_light;
	int w, h, b;
	int x, y;

	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	Border_dark = mulColor(((struct gButton *)(Object->Data))->BackGroundColor, 192);	/* BackGroundColor * 192/256 */
	Border_light = mulColor(((struct gButton *)(Object->Data))->BackGroundColor, 320);	/* BackGroundColor * 320/256 */
	GrSetGCForeground(gc1, ((struct gButton *)(Object->Data))->BackGroundColor);
	GrFillRect(w1, gc1, Object->xPos, Object->yPos, Object->Width, Object->Height);
	if(((struct gButton *)(Object->Data))->down)
		GrSetGCForeground(gc1, Border_dark);
	else
		GrSetGCForeground(gc1, Border_light);
	GrLine(w1, gc1, Object->xPos, Object->yPos, Object->xPos + Object->Width - 1, Object->yPos);
	GrLine(w1, gc1, Object->xPos, Object->yPos, Object->xPos, Object->yPos + Object->Height - 1);
	if(((struct gButton *)(Object->Data))->down)
		GrSetGCForeground(gc1, Border_light);
	else
		GrSetGCForeground(gc1, Border_dark);
	GrLine(w1, gc1, Object->xPos + Object->Width - 1, Object->yPos + Object->Height - 1, Object->xPos + Object->Width - 1, Object->yPos);
	GrLine(w1, gc1, Object->xPos + Object->Width - 1, Object->yPos + Object->Height - 1, Object->xPos, Object->yPos + Object->Height - 1);

	if(((struct gButton *)(Object->Data))->Caption == NULL) return;
	GrGetGCTextSize(gc1, ((struct gButton *)(Object->Data))->Caption, ((struct gButton *)(Object->Data))->Length, GR_TFASCII | GR_TFTOP, &w, &h, &b);
	x = Object->xPos + (Object->Width - w) / 2;
	y = Object->yPos + (Object->Height - h) / 2;
	GrSetGCForeground(gc1, ((struct gButton *)(Object->Data))->ForeGroundColor);
	GrSetGCBackground(gc1, ((struct gButton *)(Object->Data))->BackGroundColor);
	GrText(w1, gc1, x, y, ((struct gButton *)(Object->Data))->Caption, ((struct gButton *)(Object->Data))->Length, GR_TFASCII | GR_TFTOP);
}

/*
	paint function for color bars
*/
void paintColorBar(struct gObject *Object)
{
	int x;
	int l, r, rval, gval, bval;
	int mr, mg, mb;

	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	l = ((struct gColorBar *)(Object->Data))->lColor;
	r = ((struct gColorBar *)(Object->Data))->rColor;

	/* interpolate colors - mul 1024 to reduce rounding errors */
	mr = (((r & 0xFF) - (l & 0xFF)) * 1024) / Object->Width;
	mg = ((((r & 0xFF00) >> 8) - ((l & 0xFF00) >> 8)) * 1024) / Object->Width;
	mb = ((((r & 0xFF0000) >> 16) - ((l & 0xFF0000) >> 16)) * 1024) / Object->Width;
	for(x=0;x<Object->Width;x++)
	{
		rval = mr * x / 1024 + (l & 0xFF);
		gval = mg * x / 1024 + ((l & 0xFF00) >> 8);
		bval = mb * x / 1024 + ((l & 0xFF0000) >> 16);
		GrSetGCForeground(gc1, GR_RGB(rval, gval, bval));
		GrLine(w1, gc1, Object->xPos + x, Object->yPos, Object->xPos + x, Object->yPos + Object->Height - 1);
	}
}

/*
	paint functions for colored areas
*/
void paintColorArea(struct gObject *Object)
{
	int x, y;
	int ul, ur, ll, lr, redl, greenl, bluel, redr, greenr, bluer, redval, greenval, blueval;
	int mrx, mgx, mbx, mrl, mgl, mbl, mrr, mgr, mbr;
	int *pixelbuf;	/* pixel buffer */

#ifdef __PAINTCOLORAREAS__

	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	pixelbuf = NULL;
	pixelbuf = malloc(Object->Width * Object->Height * 4);
	if(pixelbuf == NULL) return;

	ul = ((struct gColorArea *)(Object->Data))->ulColor;
	ur = ((struct gColorArea *)(Object->Data))->urColor;
	ll = ((struct gColorArea *)(Object->Data))->llColor;
	lr = ((struct gColorArea *)(Object->Data))->lrColor;

	/* interpolate colors at left and right end (vertical interpolation) - mul 1024 to reduce errors */
	mrl = (((ll & 0xFF) - (ul & 0xFF)) * 1024) / Object->Height;
	mgl = ((((ll & 0xFF00) >> 8) - ((ul & 0xFF00) >> 8)) * 1024) / Object->Height;
	mbl = ((((ll & 0xFF0000) >> 16) - ((ul & 0xFF0000) >> 16)) * 1024) / Object->Height;
	mrr = (((lr & 0xFF) - (ur & 0xFF)) * 1024) / Object->Height;
	mgr = ((((lr & 0xFF00) >> 8) - ((ur & 0xFF00) >> 8)) * 1024) / Object->Height;
	mbr = ((((lr & 0xFF0000) >> 16) - ((ur & 0xFF0000) >> 16)) * 1024) / Object->Height;
	for(y=0;y<Object->Height;y++)
	{
		/* calculate left and right color components - mul 1024 to reduce erros */
		redl = mrl * y + (ul & 0xFF) * 1024;
		greenl = mgl * y + ((ul & 0xFF00) >> 8) * 1024;
		bluel = mbl * y + ((ul & 0xFF000) >> 16) * 1024;
		redr = mrr * y + (ur & 0xFF) * 1024;
		greenr = mgr * y + ((ur & 0xFF00) >> 8) * 1024;
		bluer = mbr * y + ((ur & 0xFF000) >> 16) * 1024;
		/* interpolate colors (horizontal interpolation) - left and right color already multiplicated by 1024*/
		mrx = (redr - redl) / Object->Width;
		mgx = (greenr - greenl) / Object->Width;
		mbx = (bluer - bluel) / Object->Width;
		for(x=0;x<Object->Width;x++)
		{
			/* need to correct mul 1024 */
			redval = (mrx * x + redl) / 1024;
			greenval = (mgx * x + greenl) / 1024;
			blueval = (mbx * x + bluel) / 1024;
			pixelbuf[y * Object->Width + x] = GR_RGB(redval, greenval, blueval);
		}
	}
	/* paint pixel buffer */
	GrArea(w1, gc1, Object->xPos, Object->yPos, Object->Width, Object->Height, pixelbuf, MWPF_RGB);

	free(pixelbuf);

#endif

}

/*
	event dummies - for demonstration
*/
void defaultUp(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	printf("Up   : Object = %08X\n", (int)Object);
}

void defaultDown(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	printf("Down : Object = %08X\n", (int)Object);
}

void defaultMove(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	printf("Move : Object = %08X\n", (int)Object);
}

void defaultExit(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	printf("Exit : Object = %08X\n", (int)Object);
}

/*
	event functions
*/
void ButtonDown(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	((struct gButton *)(Object->Data))->down = 1;
	if(Object->onPaint != NULL) Object->onPaint(Object);
}

void ButtonUp(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	if(((struct gButton *)(Object->Data))->down)
	{
		((struct gButton *)(Object->Data))->down = 0;
		if(Object->onPaint != NULL) Object->onPaint(Object);
		if(((struct gButton *)(Object->Data))->Action != NULL) ((struct gButton *)(Object->Data))->Action(event, Object);
	}
}

void ButtonExit(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;

	if(((struct gButton *)(Object->Data))->down)
	{
		((struct gButton *)(Object->Data))->down = 0;
		if(Object->onPaint != NULL) Object->onPaint(Object);
	}
}

void QuitAction(GR_EVENT event, struct gObject *Object)
{
	exitProgram = 1;
}

void SliderEvent(GR_EVENT event, struct gObject *Object)
{
	int Pos, Min, Max;

	if(Object == NULL) return;
	if(Object->Data == NULL) return;
	if(!(event.mouse.buttons & GR_BUTTON_L)) return;

	Min = ((struct gSlider *)(Object->Data))->Min;
	Max = ((struct gSlider *)(Object->Data))->Max;
	Pos = event.mouse.x - Object->xPos;

	if(Pos >= Object->Width - 1) return;

	((struct gSlider *)(Object->Data))->Value = (Pos * (Max - Min) / (Object->Width - 2)) + Min;

	if(Object->onPaint != NULL) Object->onPaint(Object);
}

void SliderRedEvent(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;
	if(!(event.mouse.buttons & GR_BUTTON_L)) return;
	SliderEvent(event, Object);

	/* repaint affected objects */

	if(ColorBarR == NULL) return;
	if(ColorBarR->Data == NULL) return;
	((struct gColorBar *)(ColorBarR->Data))->lColor = GR_RGB(((struct gSlider *)(Object->Data))->Value, 0 , 0);
	if(ColorBarR->onPaint != NULL) ColorBarR->onPaint(ColorBarR);

	if(ColorArea1 == NULL) return;
	if(ColorArea1->Data == NULL) return;
	((struct gColorArea *)(ColorArea1->Data))->ulColor = GR_RGB(((struct gSlider *)(Object->Data))->Value, 0 , 0);
	if(ColorArea1->onPaint != NULL) ColorArea1->onPaint(ColorArea1);

	if(ColorArea2 == NULL) return;
	if(ColorArea2->Data == NULL) return;
	((struct gColorArea *)(ColorArea2->Data))->ulColor = GR_RGB(((struct gSlider *)(Object->Data))->Value, 0 , 0);
	if(ColorArea2->onPaint != NULL) ColorArea2->onPaint(ColorArea2);
}

void SliderGreenEvent(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;
	if(!(event.mouse.buttons & GR_BUTTON_L)) return;
	SliderEvent(event, Object);

	/* repaint affected objects */
	if(ColorBarG == NULL) return;
	if(ColorBarG->Data == NULL) return;
	((struct gColorBar *)(ColorBarG->Data))->lColor = GR_RGB(0, ((struct gSlider *)(Object->Data))->Value , 0);
	if(ColorBarG->onPaint != NULL) ColorBarG->onPaint(ColorBarG);

	if(ColorArea1 == NULL) return;
	if(ColorArea1->Data == NULL) return;
	((struct gColorArea *)(ColorArea1->Data))->urColor = GR_RGB(0, ((struct gSlider *)(Object->Data))->Value , 0);
	if(ColorArea1->onPaint != NULL) ColorArea1->onPaint(ColorArea1);

	if(ColorArea2 == NULL) return;
	if(ColorArea2->Data == NULL) return;
	((struct gColorArea *)(ColorArea2->Data))->urColor = GR_RGB(0, ((struct gSlider *)(Object->Data))->Value , 0);
	if(ColorArea2->onPaint != NULL) ColorArea2->onPaint(ColorArea2);
}

void SliderBlueEvent(GR_EVENT event, struct gObject *Object)
{
	if(Object == NULL) return;
	if(Object->Data == NULL) return;
	if(!(event.mouse.buttons & GR_BUTTON_L)) return;
	SliderEvent(event, Object);

	/* repaint affected objects */
	if(ColorBarB == NULL) return;
	if(ColorBarB->Data == NULL) return;
	((struct gColorBar *)(ColorBarB->Data))->lColor = GR_RGB(0, 0, ((struct gSlider *)(Object->Data))->Value);
	if(ColorBarB->onPaint != NULL) ColorBarB->onPaint(ColorBarB);

	if(ColorArea1 == NULL) return;
	if(ColorArea1->Data == NULL) return;
	((struct gColorArea *)(ColorArea1->Data))->llColor = GR_RGB(0, 0, ((struct gSlider *)(Object->Data))->Value);
	if(ColorArea1->onPaint != NULL) ColorArea1->onPaint(ColorArea1);

	if(ColorArea2 == NULL) return;
	if(ColorArea2->Data == NULL) return;
	((struct gColorArea *)(ColorArea2->Data))->llColor = GR_RGB(0, 0, ((struct gSlider *)(Object->Data))->Value);
	if(ColorArea2->onPaint != NULL) ColorArea2->onPaint(ColorArea2);
}

/*
	control and init functions
*/

/*
	looks for an object at coordinates [x;y]
*/
int findObject(int x, int y, struct gObject **Object)
{
	struct lElement *Element;
	int match = 0;

	Element = &root;
	*Object = NULL;
	while((!match) && (Element->next != NULL))
	{
		Element = Element->next;
		if(Element->Object != NULL)
		{
			if((Element->Object->xPos <= x) && (Element->Object->yPos <= y) &&
			   (Element->Object->xPos + Element->Object->Width > x) &&
			   (Element->Object->yPos + Element->Object->Height > y))
		 	{
				match = 1;
				*Object = Element->Object;
			}
		}
	}

	return(match);
}

/*
	paints all objects
*/
void paintMenu()
{
	struct lElement *Element;
	Element = &root;

	while(Element->next != NULL)
	{
		Element = Element->next;
		if(Element->Object != NULL)
		{
			if(Element->Object->onPaint != NULL) Element->Object->onPaint(Element->Object);
		}
	}
}

/*
	builds object list and initialises objects
*/
void buildMenu()
{
	struct lElement *Element;

	/* ColorBarR */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gColorBar));
	Element->Object->Type = TYPE_COLORBAR;
	Element->Object->xPos = 88;
	Element->Object->yPos = 8;
	Element->Object->Width = 100;
	Element->Object->Height = 32;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = defaultDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintColorBar;
	((struct gColorBar *)(Element->Object->Data))->lColor = GR_COLOR_RED;
	((struct gColorBar *)(Element->Object->Data))->rColor = GR_COLOR_GRAY;
	addElement(&root, Element);
	ColorBarR = Element->Object;

	/* ColorBarG */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gColorBar));
	Element->Object->Type = TYPE_COLORBAR;
	Element->Object->xPos = 88;
	Element->Object->yPos = 48;
	Element->Object->Width = 100;
	Element->Object->Height = 32;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = defaultDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintColorBar;
	((struct gColorBar *)(Element->Object->Data))->lColor = GR_COLOR_GREEN;
	((struct gColorBar *)(Element->Object->Data))->rColor = GR_COLOR_GRAY;
	addElement(&root, Element);
	ColorBarG = Element->Object;

	/* ColorBarB */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gColorBar));
	Element->Object->Type = TYPE_COLORBAR;
	Element->Object->xPos = 88;
	Element->Object->yPos = 88;
	Element->Object->Width = 100;
	Element->Object->Height = 32;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = defaultDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintColorBar;
	((struct gColorBar *)(Element->Object->Data))->lColor = GR_COLOR_BLUE;
	((struct gColorBar *)(Element->Object->Data))->rColor = GR_COLOR_GRAY;
	addElement(&root, Element);
	ColorBarB = Element->Object;

	/* ColorArea1 */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gColorArea));
	Element->Object->Type = TYPE_COLORAREA;
	Element->Object->xPos = 88;
	Element->Object->yPos = 128;
	Element->Object->Width = 48;
	Element->Object->Height = 48;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = defaultDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintColorArea;
	((struct gColorArea *)(Element->Object->Data))->ulColor = GR_COLOR_RED;
	((struct gColorArea *)(Element->Object->Data))->urColor = GR_COLOR_GREEN;
	((struct gColorArea *)(Element->Object->Data))->llColor = GR_COLOR_BLUE;
	((struct gColorArea *)(Element->Object->Data))->lrColor = GR_COLOR_BLACK;
	addElement(&root, Element);
	ColorArea1 = Element->Object;

	/* ColorArea2 */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gColorArea));
	Element->Object->Type = TYPE_COLORAREA;
//	Element->Object->xPos = 128;
	Element->Object->xPos = 140;
	Element->Object->yPos = 128;
	Element->Object->Width = 48;
	Element->Object->Height = 48;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = defaultDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintColorArea;
	((struct gColorArea *)(Element->Object->Data))->ulColor = GR_COLOR_RED;
	((struct gColorArea *)(Element->Object->Data))->urColor = GR_COLOR_GREEN;
	((struct gColorArea *)(Element->Object->Data))->llColor = GR_COLOR_BLUE;
	((struct gColorArea *)(Element->Object->Data))->lrColor = GR_COLOR_WHITE;
	addElement(&root, Element);
	ColorArea2 = Element->Object;

	/* QuitButton */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gButton));
	Element->Object->Type = TYPE_BUTTON;
	Element->Object->xPos = 8;
	Element->Object->yPos = 8;
	Element->Object->Width = 64;
	Element->Object->Height = 32;
	Element->Object->onUp = ButtonUp;
	Element->Object->onDown = ButtonDown;
	Element->Object->onMove = defaultMove;
	Element->Object->onExit = ButtonExit;
	Element->Object->onPaint = paintButton;
	((struct gButton *)(Element->Object->Data))->Caption = "Quit";
	((struct gButton *)(Element->Object->Data))->Length = 4;
	((struct gButton *)(Element->Object->Data))->ForeGroundColor = GR_COLOR_BLACK;
	((struct gButton *)(Element->Object->Data))->BackGroundColor = GR_COLOR_GRAY;
	((struct gButton *)(Element->Object->Data))->down = 0;
	((struct gButton *)(Element->Object->Data))->Action = QuitAction;
	addElement(&root, Element);

	/* Slider red */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gSlider));
	Element->Object->Type = TYPE_SLIDER;
	Element->Object->xPos = 8;
	Element->Object->yPos = 64;
	Element->Object->Width = 64;
	Element->Object->Height = 16;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = SliderRedEvent;
	Element->Object->onMove = SliderRedEvent;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintSlider;
	((struct gSlider *)(Element->Object->Data))->Caption = NULL;
	((struct gSlider *)(Element->Object->Data))->ForeGroundColor = GR_COLOR_RED;
	((struct gSlider *)(Element->Object->Data))->BackGroundColor = GR_COLOR_GRAY;
	((struct gSlider *)(Element->Object->Data))->Min = 0;
	((struct gSlider *)(Element->Object->Data))->Max = 255;
	((struct gSlider *)(Element->Object->Data))->Value = 255;
	addElement(&root, Element);

	/* Slider green */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gSlider));
	Element->Object->Type = TYPE_SLIDER;
	Element->Object->xPos = 8;
	Element->Object->yPos = 88;
	Element->Object->Width = 64;
	Element->Object->Height = 16;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = SliderGreenEvent;
	Element->Object->onMove = SliderGreenEvent;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintSlider;
	((struct gSlider *)(Element->Object->Data))->Caption = NULL;
	((struct gSlider *)(Element->Object->Data))->ForeGroundColor = GR_COLOR_GREEN;
	((struct gSlider *)(Element->Object->Data))->BackGroundColor = GR_COLOR_GRAY;
	((struct gSlider *)(Element->Object->Data))->Min = 0;
	((struct gSlider *)(Element->Object->Data))->Max = 255;
	((struct gSlider *)(Element->Object->Data))->Value = 255;
	addElement(&root, Element);

	/* Slider blue */
	Element = malloc(sizeof(struct lElement));
	Element->Object = malloc(sizeof(struct gObject));
	Element->Object->Data = malloc(sizeof(struct gSlider));
	Element->Object->Type = TYPE_SLIDER;
	Element->Object->xPos = 8;
	Element->Object->yPos = 112;
	Element->Object->Width = 64;
	Element->Object->Height = 16;
	Element->Object->onUp = defaultUp;
	Element->Object->onDown = SliderBlueEvent;
	Element->Object->onMove = SliderBlueEvent;
	Element->Object->onExit = defaultExit;
	Element->Object->onPaint = paintSlider;
	((struct gSlider *)(Element->Object->Data))->Caption = NULL;
	((struct gSlider *)(Element->Object->Data))->ForeGroundColor = GR_COLOR_BLUE;
	((struct gSlider *)(Element->Object->Data))->BackGroundColor = GR_COLOR_GRAY;
	((struct gSlider *)(Element->Object->Data))->Min = 0;
	((struct gSlider *)(Element->Object->Data))->Max = 255;
	((struct gSlider *)(Element->Object->Data))->Value = 255;
	addElement(&root, Element);

	paintMenu();
}

/*
	delete object list and objects - not implemented yet
*/
void delMenu()
{

}

/*
	interpretes nano-X events and calls callbacks
*/
void pollMenu(GR_EVENT event)
{
	static struct gObject *last = NULL;
	struct gObject *Object = NULL;

	if(event.type != GR_EVENT_TYPE_EXPOSURE)
	{
		findObject(event.mouse.x, event.mouse.y, &Object);
		if(last != Object)
		{
			if(last != NULL)
			{
				if(last->onExit != NULL) last->onExit(event, last);
				last = NULL;
			}
		}
		last = Object;
		if(Object == NULL) return;
	}

	switch(event.type)
	{
		case GR_EVENT_TYPE_BUTTON_DOWN :	if(Object->onDown != NULL) Object->onDown(event, Object);
											break;
		case GR_EVENT_TYPE_BUTTON_UP :		if(Object->onUp != NULL) Object->onUp(event, Object);
											break;
		case GR_EVENT_TYPE_MOUSE_MOTION :	if(Object->onMove != NULL) Object->onMove(event, Object);
											break;
		case GR_EVENT_TYPE_EXPOSURE :		paintMenu();
											break;
	}
}

/*
	main event loop
*/

int main()
{
	GR_EVENT event;
	GR_CURSOR_ID cursor;
	char bits = 0;

	if (GrOpen() < 0) {
		printf("Can't open graphics\n");
		exit(1);
	}
	GrSetErrorHandler(NULL);

	w1 = GrNewWindowEx(GR_WM_PROPS_APPWINDOW, "Nano-X demo", GR_ROOT_WINDOW_ID, 0, 0, W1_WIDTH, W1_HEIGHT, GR_COLOR_GRAY);
	gc1 = GrNewGC();
	GrMapWindow(w1);
	GrSetFocus(w1);
	GrSelectEvents(w1, GR_EVENT_MASK_BUTTON_DOWN | GR_EVENT_MASK_BUTTON_UP |
	                   GR_EVENT_MASK_EXPOSURE | GR_EVENT_MASK_MOUSE_MOTION);

	buildMenu();

	cursor = GrNewCursor(1, 1, 0, 0, 0, 0, &bits, &bits);
	GrSetWindowCursor(w1, cursor);

	while(!exitProgram)
	{
		GrGetNextEvent(&event);

		pollMenu(event);
	}

	GrSetWindowCursor(w1, 0);
	GrDestroyCursor(cursor);

	delMenu();

	GrClose();

	return(0);
}
